@@ -7,7 +7,12 @@ from functools import partial
 
 from sql import Null
 
-from ..model import ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields
+from trytond.config import config
+from trytond.i18n import gettext
+from trytond.model.exceptions import ValidationError
+from ..model import (
+    ModelView, ModelStorage, ModelSQL, DeactivableMixin, fields,
+    sequence_ordered)
 from ..tools import file_open
 from ..pyson import PYSONDecoder, PYSON, Eval
 from ..transaction import Transaction
@@ -21,6 +26,31 @@ __all__ = [
     'ActionWizard', 'ActionURL',
     ]
 
+if not config.get('html', 'plugins-ir.action.report-report_content_html'):
+    config.set(
+        'html', 'plugins-ir.action.report-report_content_html', 'fullpage')
+
+
+class WizardModelError(ValidationError):
+    pass
+
+
+class EmailError(ValidationError):
+    pass
+
+
+class ViewError(ValidationError):
+    pass
+
+
+class DomainError(ValidationError):
+    pass
+
+
+class ContextError(ValidationError):
+    pass
+
+
 EMAIL_REFKEYS = set(('cc', 'to', 'subject'))
 
 
@@ -109,10 +139,8 @@ class ActionKeyword(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(ActionKeyword, cls).__setup__()
-        cls.__rpc__.update({'get_keyword': RPC()})
-        cls._error_messages.update({
-                'wrong_wizard_model': ('Wrong wizard model in keyword action '
-                    '"%s".'),
+        cls.__rpc__.update({
+                'get_keyword': RPC(cache=dict(days=1)),
                 })
 
     @classmethod
@@ -147,8 +175,9 @@ class ActionKeyword(ModelSQL, ModelView):
                 if action_wizard.model:
                     if not str(self.model).startswith(
                             '%s,' % action_wizard.model):
-                        self.raise_user_error('wrong_wizard_model', (
-                                action_wizard.rec_name,))
+                        raise WizardModelError(
+                            gettext('ir.msg_action_wrong_wizard_model',
+                                name=action_wizard.rec_name))
 
     @staticmethod
     def _convert_vals(vals):
@@ -397,6 +426,14 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
         'get_report_content', setter='set_report_content')
     report_content_name = fields.Function(fields.Char('Content Name'),
         'on_change_with_report_content_name')
+    report_content_html = fields.Function(fields.Binary(
+            "Content HTML",
+            states={
+                'invisible': ~Eval('template_extension').in_(
+                    ['html', 'xhtml']),
+                },
+            depends=['template_extension']),
+        'get_report_content_html', setter='set_report_content_html')
     action = fields.Many2One('ir.action', 'Action', required=True,
             ondelete='CASCADE')
     direct_print = fields.Boolean('Direct Print')
@@ -488,13 +525,6 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
         "Example: {'to': 'test@example.com', 'cc': 'user@example.com'}")
     pyson_email = fields.Function(fields.Char('PySON Email'), 'get_pyson')
 
-    @classmethod
-    def __setup__(cls):
-        super(ActionReport, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_email': 'Invalid email definition on report "%s".',
-                })
-
     @classmethod
     def __register__(cls, module_name):
         super(ActionReport, cls).__register__(module_name)
@@ -562,10 +592,13 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
                 if isinstance(value, dict):
                     inkeys = set(value)
                     if not inkeys <= EMAIL_REFKEYS:
-                        cls.raise_user_error('invalid_email', (
-                                report.rec_name,))
+                        raise EmailError(
+                            gettext('ir.msg_report_invalid_email',
+                                name=report.rec_name))
                 else:
-                    cls.raise_user_error('invalid_email', (report.rec_name,))
+                    raise EmailError(
+                        gettext('ir.msg_report_invalid_email',
+                            name=report.rec_name))
 
     def get_is_custom(self, name):
         return bool(self.report_content_custom)
@@ -597,6 +630,16 @@ class ActionReport(ActionMixin, ModelSQL, ModelView):
     def set_report_content(cls, records, name, value):
         cls.write(records, {'%s_custom' % name: value})
 
+    @classmethod
+    def get_report_content_html(cls, reports, name):
+        return cls.get_report_content(reports, name[:-5])
+
+    @classmethod
+    def set_report_content_html(cls, reports, name, value):
+        if value is not None:
+            value = value.encode('utf-8')
+        cls.set_report_content(reports, name[:-5], value)
+
     @fields.depends('name', 'template_extension')
     def on_change_with_report_content_name(self, name=None):
         return ''.join(
@@ -677,14 +720,6 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(ActionActWindow, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_views': ('Invalid view "%(view)s" for action '
-                    '"%(action)s".'),
-                'invalid_domain': ('Invalid domain or search criteria '
-                    '"%(domain)s" on action "%(action)s".'),
-                'invalid_context': ('Invalid context "%(context)s" on action '
-                    '"%(action)s".'),
-                })
         cls.__rpc__.update({
                 'get': RPC(),
                 })
@@ -736,28 +771,28 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 for act_window_view in action.act_window_views:
                     view = act_window_view.view
                     if view.model != action.res_model:
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
                     if view.type == 'board':
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
             else:
                 for act_window_view in action.act_window_views:
                     view = act_window_view.view
                     if view.model:
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
                     if view.type != 'board':
-                        cls.raise_user_error('invalid_views', {
-                                'view': view.rec_name,
-                                'action': action.rec_name,
-                                })
+                        raise ViewError(
+                            gettext('ir.msg_action_invalid_views',
+                                view=view.rec_name,
+                                action=action.rec_name))
 
     @classmethod
     def check_domain(cls, actions):
@@ -769,29 +804,29 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 try:
                     value = PYSONDecoder().decode(domain)
                 except Exception:
-                    cls.raise_user_error('invalid_domain', {
-                            'domain': domain,
-                            'action': action.rec_name,
-                            })
+                    raise DomainError(
+                        gettext('ir.msg_action_invalid_domain',
+                            domain=domain,
+                            action=action.rec_name))
                 if isinstance(value, PYSON):
                     if not value.types() == set([list]):
-                        cls.raise_user_error('invalid_domain', {
-                                'domain': domain,
-                                'action': action.rec_name,
-                                })
+                        raise DomainError(
+                            gettext('ir.msg_action_invalid_domain',
+                                domain=domain,
+                                action=action.rec_name))
                 elif not isinstance(value, list):
-                    cls.raise_user_error('invalid_domain', {
-                            'domain': domain,
-                            'action': action.rec_name,
-                            })
+                    raise DomainError(
+                        gettext('ir.msg_action_invalid_domain',
+                            domain=domain,
+                            action=action.rec_name))
                 else:
                     try:
                         fields.domain_validate(value)
                     except Exception:
-                        cls.raise_user_error('invalid_domain', {
-                                'domain': domain,
-                                'action': action.rec_name,
-                                })
+                        raise DomainError(
+                            gettext('ir.msg_action_invalid_domain',
+                                domain=domain,
+                                action=action.rec_name))
 
     @classmethod
     def check_context(cls, actions):
@@ -801,29 +836,29 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
                 try:
                     value = PYSONDecoder().decode(action.context)
                 except Exception:
-                    cls.raise_user_error('invalid_context', {
-                            'context': action.context,
-                            'action': action.rec_name,
-                            })
+                    raise ContextError(
+                        gettext('ir.msg_action_invalid_context',
+                            context=action.context,
+                            action=action.rec_name))
                 if isinstance(value, PYSON):
                     if not value.types() == set([dict]):
-                        cls.raise_user_error('invalid_context', {
-                                'context': action.context,
-                                'action': action.rec_name,
-                                })
+                        raise ContextError(
+                            gettext('ir.msg_action_invalid_context',
+                                context=action.context,
+                                action=action.rec_name))
                 elif not isinstance(value, dict):
-                    cls.raise_user_error('invalid_context', {
-                            'context': action.context,
-                            'action': action.rec_name,
-                            })
+                    raise ContextError(
+                        gettext('ir.msg_action_invalid_context',
+                            context=action.context,
+                            action=action.rec_name))
                 else:
                     try:
                         fields.context_validate(value)
                     except Exception:
-                        cls.raise_user_error('invalid_context', {
-                                'context': action.context,
-                                'action': action.rec_name,
-                                })
+                        raise ContextError(
+                            gettext('ir.msg_action_invalid_context',
+                                context=action.context,
+                                action=action.rec_name))
 
     def get_views(self, name):
         return [(view.view.id, view.view.type)
@@ -860,19 +895,23 @@ class ActionActWindow(ActionMixin, ModelSQL, ModelView):
         return Action.get_action_values(cls.__name__, [action_id])[0]
 
 
-class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
+class ActionActWindowView(
+        sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     "Action act window view"
     __name__ = 'ir.action.act_window.view'
-    sequence = fields.Integer('Sequence', required=True)
     view = fields.Many2One('ir.ui.view', 'View', required=True,
             ondelete='CASCADE')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
             ondelete='CASCADE')
 
     @classmethod
-    def __setup__(cls):
-        super(ActionActWindowView, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
+    def __register__(cls, module_name):
+        super().__register__(module_name)
+
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: remove required on sequence
+        table.not_null_action('sequence', 'remove')
 
     @classmethod
     def create(cls, vlist):
@@ -894,24 +933,24 @@ class ActionActWindowView(DeactivableMixin, ModelSQL, ModelView):
         pool.get('ir.action.keyword')._get_keyword_cache.clear()
 
 
-class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
+class ActionActWindowDomain(
+        sequence_ordered(), DeactivableMixin, ModelSQL, ModelView):
     "Action act window domain"
     __name__ = 'ir.action.act_window.domain'
     name = fields.Char('Name', translate=True)
-    sequence = fields.Integer('Sequence', required=True)
     domain = fields.Char('Domain')
     count = fields.Boolean('Count')
     act_window = fields.Many2One('ir.action.act_window', 'Action',
         select=True, required=True, ondelete='CASCADE')
 
     @classmethod
-    def __setup__(cls):
-        super(ActionActWindowDomain, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
-        cls._error_messages.update({
-                'invalid_domain': ('Invalid domain or search criteria '
-                    '"%(domain)s" on action "%(action)s".'),
-                })
+    def __register__(cls, module_name):
+        super().__register__(module_name)
+
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: remove required on sequence
+        table.not_null_action('sequence', 'remove')
 
     @classmethod
     def default_count(cls):
@@ -942,10 +981,10 @@ class ActionActWindowDomain(DeactivableMixin, ModelSQL, ModelView):
                 except Exception:
                     value = None
             if value is None:
-                cls.raise_user_error('invalid_domain', {
-                        'domain': action.domain,
-                        'action': action.rec_name,
-                        })
+                raise DomainError(gettext(
+                        'ir.msg_action_invalid_domain',
+                        domain=action.domain,
+                        action=action.rec_name))
 
     @classmethod
     def create(cls, vlist):
