@@ -3,16 +3,15 @@
 import warnings
 from functools import wraps
 
-import sql
 from sql import (operators, Column, Literal, Select, CombiningQuery, Null,
     Query, Expression, Cast)
-from sql.aggregate import Min
 from sql.conditionals import Coalesce, NullIf
 from sql.operators import Concat
 
 from trytond import backend
-from trytond.pyson import PYSON, PYSONEncoder, Eval
+from trytond.pyson import PYSON, PYSONEncoder, PYSONDecoder, Eval
 from trytond.const import OPERATORS
+from trytond.tools.string_ import StringPartitioned
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.cache import LRUDictTransaction
@@ -21,8 +20,6 @@ from ...rpc import RPC
 
 Database = backend.get('Database')
 
-_sql_version = tuple(map(int, sql.__version__.split('.')))
-
 
 def domain_validate(value):
     assert isinstance(value, list), 'domain must be a list'
@@ -49,9 +46,11 @@ def domain_validate(value):
 
 def states_validate(value):
     assert isinstance(value, dict), 'states must be a dict'
+    assert set(value).issubset({'required', 'readonly', 'invisible'}), (
+        'extra keys "%(keys)s" in states' % {
+            'keys': set(value) - {'required', 'readonly', 'invisible'},
+            })
     for state in value:
-        if state == 'icon':
-            continue
         assert isinstance(value[state], (bool, PYSON)), \
             'values of states must be PYSON'
         if hasattr(value[state], 'types'):
@@ -90,10 +89,7 @@ def _set_value(record, field):
     if field.startswith('_parent_'):
         field = field[8:]  # Strip '_parent_'
     if not hasattr(record, field):
-        default = None
-        if hasattr(record, '_defaults') and field in record._defaults:
-            default = record._defaults[field]()
-        setattr(record, field, default)
+        setattr(record, field, None)
     elif nested:
         parent = getattr(record, field)
         if parent:
@@ -168,6 +164,21 @@ def instanciate_values(Target, value):
     return tuple(instance(x) for x in (value or []))
 
 
+def instantiate_context(field, record):
+    from ..modelstorage import EvalEnvironment
+    ctx = {}
+    if field.context:
+        pyson_context = PYSONEncoder().encode(field.context)
+        ctx.update(PYSONDecoder(
+                EvalEnvironment(record, record.__class__)).decode(
+                pyson_context))
+    datetime_ = None
+    if getattr(field, 'datetime_field', None):
+        datetime_ = getattr(record, field.datetime_field, None)
+        ctx = {'_datetime': datetime_}
+    return ctx
+
+
 def on_change_result(record):
     return record._changed_values
 
@@ -260,6 +271,22 @@ class Field(object):
         self.loading = loading
         self.name = None
 
+    @property
+    def string(self):
+        return self.__string
+
+    @string.setter
+    def string(self, value):
+        self.__string = StringPartitioned(value)
+
+    @property
+    def help(self):
+        return self.__help
+
+    @help.setter
+    def help(self, value):
+        self.__help = StringPartitioned(value)
+
     def _get_domain(self):
         return self.__domain
 
@@ -391,52 +418,86 @@ class Field(object):
             model.__rpc__.setdefault(
                 func_name, RPC(instantiate=0, result=result))
 
+    def definition(self, model, language):
+        pool = Pool()
+        Translation = pool.get('ir.translation')
+        encoder = PYSONEncoder()
+        definition = {
+            'context': encoder.encode(self.context),
+            'loading': self.loading,
+            'name': self.name,
+            'on_change': list(self.on_change),
+            'on_change_with': list(self.on_change_with),
+            'readonly': self.readonly,
+            'required': self.required,
+            'states': encoder.encode(self.states),
+            'type': self._type,
+            'domain': encoder.encode(self.domain),
+            'searchable': hasattr(model, 'search'),
+            'sortable': hasattr(model, 'search'),
+            }
+
+        # Add id to on_change's if they are not cached
+        # Not having the id increase the efficiency of the cache
+        for method in ['on_change', 'on_change_with']:
+            changes = definition[method]
+            if changes:
+                method_name = method + '_' + self.name
+                if not model.__rpc__[method_name].cache:
+                    changes.append('id')
+
+        name = '%s,%s' % (model.__name__, self.name)
+        for attr, ttype in [('string', 'field'), ('help', 'help')]:
+            definition[attr] = ''
+            for source in getattr(self, attr):
+                definition[attr] += (
+                    Translation.get_source(name, ttype, language, source)
+                    or source)
+        return definition
+
+    def definition_translations(self, model, language):
+        "Returns sources used for definition"
+        name = '%s,%s' % (model.__name__, self.name)
+        translations = []
+        for attr, ttype in [('string', 'field'), ('help', 'help')]:
+            for source in getattr(self, attr):
+                translations.append((name, ttype, language, source))
+        return translations
+
 
 class FieldTranslate(Field):
 
     def _get_translation_join(self, Model, name,
             translation, model, table, from_, language):
-        if backend.name() == 'postgresql' and _sql_version >= (1, 1, 0):
-            query = translation.select(
-                translation.res_id.as_('res_id'),
-                translation.value.as_('value'),
-                translation.name.as_('name'),
-                distinct=True,
-                distinct_on=[translation.res_id, translation.name],
-                order_by=[
-                    translation.res_id,
-                    translation.name,
-                    translation.id.desc])
-        else:
-            query = translation.select(
-                translation.res_id.as_('res_id'),
-                Min(translation.value).as_('value'),
-                translation.name.as_('name'),
-                group_by=[translation.res_id, translation.name])
         if Model.__name__ == 'ir.model':
-            name_ = Concat(Concat(table.model, ','), name)
-            type_ = 'model'
-            res_id = -1
+            return from_.join(translation, 'LEFT',
+                condition=(translation.name == Concat(Concat(
+                            table.model, ','), name))
+                & (translation.res_id == -1)
+                & (translation.lang == language)
+                & (translation.type == 'model')
+                & (translation.fuzzy == False))
         elif Model.__name__ == 'ir.model.field':
-            from_ = from_.join(model, 'LEFT',
-                condition=model.id == table.model)
-            name_ = Concat(Concat(model.model, ','), table.name)
             if name == 'field_description':
                 type_ = 'field'
             else:
                 type_ = 'help'
-            res_id = -1
+            return from_.join(model, 'LEFT',
+                condition=model.id == table.model).join(
+                    translation, 'LEFT',
+                    condition=(translation.name == Concat(Concat(
+                                model.model, ','), table.name))
+                    & (translation.res_id == -1)
+                    & (translation.lang == language)
+                    & (translation.type == type_)
+                    & (translation.fuzzy == False))
         else:
-            name_ = '%s,%s' % (Model.__name__, name)
-            type_ = 'model'
-            res_id = table.id
-        query.where = (
-            (translation.lang == language)
-            & (translation.type == type_)
-            & (translation.fuzzy == Literal(False))
-            )
-        return query, from_.join(query, 'LEFT',
-            condition=(query.res_id == res_id) & (query.name == name_))
+            return from_.join(translation, 'LEFT',
+                condition=(translation.res_id == table.id)
+                & (translation.name == '%s,%s' % (Model.__name__, name))
+                & (translation.lang == language)
+                & (translation.type == 'model')
+                & (translation.fuzzy == False))
 
     def convert_domain(self, domain, tables, Model):
         from trytond.ir.lang import get_parent_language
@@ -454,7 +515,7 @@ class FieldTranslate(Field):
         column = None
         while language:
             translation = Translation.__table__()
-            translation, join = self._get_translation_join(
+            join = self._get_translation_join(
                 Model, name, translation, model, table, join, language)
             column = Coalesce(NullIf(column, ''), translation.value)
             language = get_parent_language(language)
@@ -490,7 +551,7 @@ class FieldTranslate(Field):
             if key not in tables:
                 translation = Translation.__table__()
                 model = IrModel.__table__()
-                translation, join = self._get_translation_join(
+                join = self._get_translation_join(
                     Model, name, translation, model, table, table, language)
                 if join.left == table:
                     tables[key] = {
@@ -511,4 +572,9 @@ class FieldTranslate(Field):
             column = Coalesce(NullIf(column, ''), translation.value)
             language = get_parent_language(language)
 
-        return [Coalesce(NullIf(column, ''), self.sql_column(table))]
+        return [Coalesce(column, self.sql_column(table))]
+
+    def definition(self, model, language):
+        definition = super().definition(model, language)
+        definition['translate'] = self.translate
+        return definition
